package YMLD;
use strict;
use warnings;

BEGIN {
    use Exporter;
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK);
    $VERSION = 1.00;
    @ISA = qw{Exporter};
    # These are the functions we want to export from our module
    @EXPORT_OK = qw { 
        db_connect
        display_performances
        display_productions
        display_seat_info
        last_insert_id
        select_available_seats
        select_performance_data
        select_productions
        select_seat_info
    };
}

our @EXPORT_OK;

########################################
# Define YMLD subroutines
########################################

sub db_connect {
    my $database = 'YMLD';
    my $user = 'userid';
    my $password = 'password';

    # Connect to the database
    my $dbh = DBI->connect("dbi:DB2:$database", $user, $password,
        { AutoCommit => 1,
        RaiseError => 0 });

    return $dbh;
}

sub select_productions {
  my ($dbh, $results) = @_;

  my $select = q{
    SELECT Production_No, Production_Title,
    Production_StartDate, Production_EndDate
    FROM Productions
  };
  my $sth = $dbh->prepare($select);

  my $rc = $sth->execute();
  my ($id, $title, $start, $end);
  $rc = $sth->bind_columns(\($id, $title, $start, $end));
  while ($sth->fetch()) {
      push(@$results, [$id, $title, $start, $end]);
  }

  $sth->finish();
}

sub select_performance_data {
  my ($dbh, $performance, $results) = @_;

  my $select = q{
    SELECT Performance_No, Production_Title,
    Performance_Date, Performance_Time
    FROM Productions INNER JOIN Performances
    ON Productions.Production_No = Performances.Production_No
    WHERE Productions.Production_No = ?
    AND Performances.Performance_SeatsAve = 1
  };
  my $sth = $dbh->prepare($select);

  my $rc = $sth->execute($performance);
  my ($id, $title, $date, $time);
  $rc = $sth->bind_columns(\($id, $title, $date, $time));
  while ($sth->fetch()) {
      push(@$results, [$id, $title, $date, $time]);
  }

  $sth->finish();
}

sub select_available_seats {
  my ($dbh, $performances, $seats) = @_;

  my $select = q{
    SELECT Seat_No, CAST(PricePlan_Cost AS DECIMAL(5, 2))
    FROM Seats INNER JOIN PricePlan
    ON Seats.PricePlan_No = PricePlan.PricePlan_No
    WHERE Performance_No = ?
    AND Seats.Seat_Available = 1
  };
  my $sth = $dbh->prepare($select);

  foreach my $show (@$performances) {
      my $rc = $sth->execute($show->[0]);
      my ($id, $price);
      $rc = $sth->bind_columns(\($id, $price));
      while ($sth->fetch()) {
          $seats->{$show->[0]}{$id} = $price;
      }
  }
  $sth->finish();
}

sub display_productions {
    my ($cgi, $productions) = @_;

    print $cgi->start_table();
    print $cgi->Tr(
        $cgi->th('Title'),
        $cgi->th('Start date'),
        $cgi->th('End date'),
    );
    foreach my $show (@$productions) {
        $cgi->param(-name=>'page', -value=>'show');
        $cgi->param(-name=>'id', -value=>$show->[0]);
        print $cgi->Tr(
            $cgi->td($cgi->a({-href=>$cgi->self_url},
                    $show->[1])),
            $cgi->td($show->[2]),
            $cgi->td($show->[3]),
        );
        print "\n";
    }
    print $cgi->end_table();
}

sub display_performances {
    my ($cgi, $performances, $seats) = @_;

    print $cgi->start_table();
    print $cgi->Tr(
        $cgi->th('Title'),
        $cgi->th('Date'),
        $cgi->th('Time'),
        $cgi->th('Seats'),
    );
    foreach my $show (@$performances) {
        my @seat_list = $seats->{$show->[0]};
        $cgi->param(-name=>'page', -value=>'view');
        print $cgi->Tr(
            $cgi->td($show->[1]),
            $cgi->td($show->[2]),
            $cgi->td($show->[3]),
            $cgi->td($cgi->start_form({-method=>'POST',
                        -action=>$cgi->self_url}), 
                $cgi->popup_menu(-name=>'seat',
                    -values=>$seats->{$show->[0]},
                    -default=>$seat_list[0]
                ),
                $cgi->submit({-value=>'Display'}),
                $cgi->end_form()
            )
        );
        print "\n";
    }
    print $cgi->end_table();
}

sub display_seat_info {
    my ($cgi, $seat_info) = @_;

    print $cgi->start_table();
    print $cgi->Tr(
        $cgi->th('Seat'),
        $cgi->th('Price'),
        $cgi->th('Available'),
    );
    foreach my $seat (@$seat_info) {
        $cgi->param(-name=>'page', -value=>'view');
        $cgi->param(-name=>'id', -value=>$seat->[0]);
        print $cgi->Tr(
            $cgi->td($cgi->a({-href=>$cgi->self_url},
                    $seat->[1])),
            $cgi->td($seat->[2]),
            $cgi->td($seat->[3]),
        );
        print "\n";
    }
    print $cgi->end_table();
}

sub last_insert_id {
    my ($dbh) = @_;

    my $id = 0;
    my $get_identity = q{
        VALUES identity_val_local()
    };

    my $id_sth = $dbh->prepare($get_identity);
    $id_sth->execute();
    $id_sth->bind_col(1, \$id);
    $id_sth->fetch();
    $id_sth->finish();

    return $id;
}

# Every module must return true
1;
