#!/usr/bin/perl -w
use strict;
use warnings;
use DBI qw{ :sql_types };
use CGI qw{ *table };
use YMLD qw{
    db_connect
    display_performances
    display_productions
    display_seat_info
    select_available_seats
    select_performance_data
    select_productions
    select_seat_info
};

my $cgi = new CGI;
print $cgi->header();

my $page = $cgi->param('page');

if (!$page) {
    index_page();
}
elsif ($page eq 'show') {
    display_page();
}
elsif ($page eq 'view') {
    view_page();
}
elsif ($page eq 'search') {
    search_page();
}
else {
    unknown_page();
}

exit;

########################################
# Define subroutines
########################################

sub index_page {
    print $cgi->start_html(-title=>'YourMamaLovesDrama: Home');
    print $cgi->h1('YourMamaLovesDrama: Home');
    my $dbh = db_connect();
    my @productions =  qw{};
    select_productions($dbh, \@productions);
    display_productions($cgi, \@productions);
    $dbh->disconnect();
    print $cgi->end_html();
}

sub display_page {
    print $cgi->start_html(-title=>'YourMamaLovesDrama:
        Display performance');
    print $cgi->h1('YourMamaLovesDrama: Display performance');
    my $id = $cgi->param('id');
    if ($id =~ m/^(\d+)$/) {
        my $dbh = db_connect();
        my @performances = qw{};
        my %seats;
        select_performance_data($dbh, $id, \@performances);
        select_available_seats($dbh, \@performances, \%seats);
        display_performances($cgi, \@performances, \%seats);
        $dbh->disconnect();
    }
    else {
        print $cgi->p('Requested performance does not exist');
    }
    print $cgi->end_html();
}

sub view_page {
    print $cgi->start_html(-title=>'YourMamaLovesDrama:
        View seat information');
    print $cgi->h1('YourMamaLovesDrama: View seat information');
    my $id = $cgi->param('id');
    my $seat = $cgi->param('seat');
    my $dbh = db_connect();
    my @seat_info = qw{};
    select_seat_info($dbh, $id, $seat, \@seat_info);
    display_seat_info($cgi, \@seat_info);
    $dbh->disconnect();
    print $cgi->end_html();
}

sub search_page {
    print $cgi->start_html(-title=>'YourMamaLovesDrama:
        Search for performance');
    print $cgi->h1('YourMamaLovesDrama:
        Search for performance');
    my $dbh = db_connect();
    $dbh->disconnect();
    print $cgi->end_html();
}

sub unknown_page {
    print $cgi->start_html(-title=>'YourMamaLovesDrama:
        Unknown page');
    print $cgi->h1('YourMamaLovesDrama: Unknown page');
    print $cgi->p('The page you have requested does not exist.
        Please select your desired destination from the menu.');
    print $cgi->end_html();
}


