#!/usr/bin/perl -w
use warnings;
use strict;
use Tk;
use Tk::Table;
use DBI qw{ :sql_types };
use YMLD qw{ db_connect };

my $dbh = db_connect();
if ($dbh->errstr) {
    print $dbh->errstr;
    exit;
}
my $main = display_productions($dbh);

MainLoop;

$dbh->disconnect();
exit;

sub display_productions {
  my ($dbh) = @_;

  my $main = MainWindow->new(-title=>'YourMamaLovesDrama ' .
      'Administration Panel');
  my @columns = ('Title', 'Start Date', 'End Date');
  my @select_data;
  select_productions($dbh, \@select_data);
  my $row_count = scalar(@select_data);

  my $menuitems = [
      [Cascade=>"~File", -menuitems=>[
        [Button=>"~Refresh", -command=>[\&refresh, $main, $dbh]],
        [Button=>"~Exit", -command=>[\&exit]],
      ]],
      [Cascade=>"~Productions", -menuitems=>[
        [Button=>"~Add", -command=>[\&add_production, $main]],
      ]]
  ];

  my $menubar = $main->Menu(-menuitems=>$menuitems);
  $main->configure(-menu=>$menubar);

  my $table = $main->Table(-rows=>$row_count,
      -columns=>scalar(@columns) + 1, -scrollbars=>'se',
      -fixedrows=>1, -fixedcolumns=>1, -takefocus=>1)->pack;

  foreach my $column (1 .. scalar(@columns)) {
    my $column_header = $main->Button(
        -text=>$columns[$column - 1]
    );
    $table->put(0, $column, $column_header);
  }

  foreach my $row (1 .. $row_count) {
    my $row_header = $main->Button(-text=>"Row $row");
    $table->put($row, 0, $row_header);
    foreach my $column (1 .. scalar(@columns)) {
#Label objects are read-only, while Entry objects are editable
      my $cell = $main->Entry(-width=>20,
          -text=>$select_data[$row - 1][$column - 1]
      );
      $table->put($row, $column, $cell);
    }
  }
  return $main;
}

sub refresh {
  my ($main, $dbh) = @_;
  $main->destroy();
  display_productions($dbh);
}

sub add_production {
  my ($main) = @_;
  my $dialog = $main->Toplevel(
      -title=>'Add Production',
      );
  my $top = $dialog->Frame()->pack(-side=>'top');
  my $bottom = $dialog->Frame()->pack(-side=>'bottom');

  my $labels = $top->Frame()->pack(-side=>'left');
  my $entries = $top->Frame()->pack(-side=>'right');

  $labels->Label(-text=>'Title:')->pack(
      -side=>'top', -anchor=>'e'
  );
  my $title = $entries->Entry(-width=>20,
      -text=>'Title')->pack(-anchor=>'w'
  );
  $labels->Label(-text=>'Start date:')->pack(
      -side=>'top', -anchor=>'e'
  );
  my $start = $entries->Entry(-width=>10,
      -text=>'Start date')->pack(-anchor=>'w'
  );
  $labels->Label(-text=>'End date:')->pack(
      -side=>'top', -anchor=>'e'
  );
  my $end = $entries->Entry(-width=>10,
      -text=>'End date')->pack(-anchor=>'w'
  );

  $bottom->Button(-text=>"OK", -command=>[\&insert_production,
      $main, $dialog, $title, $start, $end])->pack(
      -side=>'left'
  );
  $bottom->Button(-text=>"Cancel", -command=>[\&dismiss,
      $dialog])->pack(-side=>'right'
  );
}

sub dismiss {
  my ($dialog) = @_;
  $dialog->destroy();
}

sub insert_production {
  my ($main, $dialog, $title, $start, $end) = @_;

  my $insert_production = q{
    INSERT INTO Productions (Production_Title,
      Production_StartDate, Production_EndDate)
      VALUES (?, ?, ?)
  };
  my $sth = $dbh->prepare($insert_production);
  $sth->bind_param(1, $title->get(), SQL_VARCHAR);
  $sth->bind_param(2, $start->get(), SQL_DATE);
  $sth->bind_param(3, $end->get(), SQL_DATE);
  my $rows_affected = $sth->execute();
  my $rc = $sth->errstr;
  if ($rc) {
    my $msg = $main->Toplevel(-title=>"Error message");
    $msg->Message(-text=>'An error occurred inserting ' .
        "the new production:\n\n$rc")->pack();
  }
  else {
    $dialog->destroy();
    refresh($main, $dbh);
  }
  $sth->finish();
}

sub select_productions {
  my ($dbh, $results) = @_;

  my $select = q{
    SELECT Production_Title, Production_StartDate,
      Production_EndDate
      FROM Productions
      ORDER BY Production_StartDate
  };
  my $sth = $dbh->prepare($select);

  my $rc = $sth->execute();
  my ($title, $start, $end);
  $rc = $sth->bind_columns(\($title, $start, $end));
  while ($sth->fetch()) {
    push(@$results, [$title, $start, $end]);
  }

  $sth->finish();
}
